<?php

// $Id: utils.php,v 1.5.2.7 2009/02/11 12:49:44 theskp Exp $

/**
 * Checks for and replaces a hexidecimal ([#]rrggbb | [#]rrggbbaa) color with its array representation
 * If $image is supplied, the color will be allocated to it, and returned
 * @param resource $image [optional] image to allocate color to
 * @param array/string $color_to_be
 * @return array representation of color: array($red, $green, $blue, $alpha)
 */
function _color($color_to_be, $image = NULL){
  
  // If it's a color already
  if (is_numeric($color_to_be)) {
    return $color;
  }
  
  // if it's already an array, don't bother converting
  if (is_array($color_to_be)){
    $rgba = $color_to_be;
  } else {
    
    $rgba = array(0, 0, 0, 0);
    if (is_string($color_to_be) && eregi('^\#?[a-f0-9]{6,8}$', $color_to_be)){
      // drop the #
      $color_to_be = substr($color_to_be, 1);
      
      if (strlen($color_to_be) == 6){
        sscanf($color_to_be, "%2x%2x%2x", $r, $g, $b);
        $a = 0;
      } else if (strlen($color_to_be) == 8) {
        sscanf($color_to_be, "%2x%2x%2x%2x", $r, $g, $b, $a);
        $a = $a > 127 ? 127 : $a;
      }
      $rgba = array($r, $g, $b, $a);
    }
  }
  
  if ($image) {
    return imagecolorresolvealpha($image, $rgba[0], $rgba[1], $rgba[2], $rgba[3]);
  }

  return $rgba;
}



/**
 * Replaces functions and [handles] within an argument with computed values and $variable['handle'] values,
 * and evaluates any contained expressions
 */
function _compute($argument, $variables){
  $substituted = _substitute($argument, $variables);
  if (ereg('[0-9\ \+\-\*\/\(\)\.]', $substituted)){
    return eval('return '.$substituted.';');
  }
}

/**
 * Replaces functions and [handles] within an argument with computed values and $variable['handle'] values
 */
function _substitute($argument, $variables){
  if (is_string($argument)) {
    $processed = $argument;
    
    /* Substitute image heights */
    while ($start = eregi('width\([^\(]+\)', $processed, $search_result)){
      $variableKey = substr($search_result[0], 6, strlen($search_result) - 6);
      $processed = str_ireplace($search_result[0], (String) imagesx($variables[$variableKey]), $processed);
    }
    /* Substitute image widths */    
    while ($start = eregi('height\([^\(]+\)', $processed, $search_result)){
      $variableKey = substr($search_result[0], 7, strlen($search_result) - 6);
      $processed = str_ireplace($search_result[0], (String) imagesy($variables[$variableKey]), $processed);
    }
    /* Substitute variables */
    while ($start = ereg('\[[^\[]+\]', $processed, $search_result)){
      $variableKey = substr($search_result[0], 1, strpos($search_result[0], ']') - 1);
      
      if (is_resource($variables[$variableKey])){
        return $variables[$variableKey];
      } else {
        $processed = str_ireplace($search_result[0], $variables[$variableKey], $processed);
      }
    }
    return $processed;
  }
  return $argument;
}



/**
 * Resolves whether a resulting image should be returned or stored (and stores it if needs be)
 */
function _resolve_destination($result, $args, &$variables, $resulting = 'as'){
  // If the resulting image should be stored as a different image
  if ($args[$resulting]){
    if (is_resource($variables[$args[$resulting]])){
      imagedestroy($variables[$args[$resulting]]);
    }
    $variables[$args[$resulting]] = $result;
    return;
  }
  // If a variable key was provided as source, replace the source with the resulting image
  if ($args['image'] && is_string($args['image'])) {
    $key = substr($args['image'], 1, strlen($args['image']) - 2);
    if (is_resource($variables[$key])){
      imagedestroy($variables[$key]);
    }
    $variables[$key] = $result;
    return;
  }
  // If the canvas is to be manipulated
  if ($source = $variables['canvas']) {
    if (is_resource($variables[$args['canvas']])){
      imagedestroy($variables[$args['canvas']]);
    }
    $variables['canvas'] = $result;
    return;
  }
  return $result;
}



/**
 * Strips the square brackets from an image handle
 *
 * @param unknown_type $string
 * @return unknown
 */
function _strip_brackets($string){
  return ereg_replace('\[|\]', '', $string);
}



/**
 * Loads a chain from the theme or modules
 *
 * @param $chain_name
 *  name of the chain to load
 * @param $args
 *  arguments to build the chain with
 * @param $arg_hash
 *  argument hash used for the chain cache
 * @return
 *  populated chain
 */
function _get_chain($chain_name, $args, $arg_hash){

  global $theme;
  global $chains;
  
  // Check the theme cache
  if ($chains['theme'][$chain_name][$arg_hash] && $arg_hash){
    return $chains['theme'][$chain_name][$arg_hash];
  }
  // Check in the theme
  $theme_hook = $theme.'_chain';
  if (function_exists($theme_hook)) {
    if ($chain = $theme_hook('load', $chain_name, $args)) {
      $chains['theme'][$chain_name][$arg_hash] = $chain;
      return $chain;
    }
  }
  
  // Check the module cache
  if ($chains['modules'][$chain_name][$arg_hash] && $arg_hash){
    return $chains['modules'][$chain_name][$arg_hash];
  }
  // Check in the modules
  if ($modules = module_implements('chain')){
    foreach($modules as $module){
      if ($chain = module_invoke($module, 'chain', 'load', $chain_name, $args)){
        $chains['modules'][$chain_name][$arg_hash] = $chain;
        return $chain;
      }
    }
  }
  
  return FALSE;
}



/**
 * Constructs a list of theme and module provided chains
 * 
 * @param $applies_to (text | image | FALSE)
 *  Text: will construct a list of all chains that apply to text
 *  Image will construct a list of all chains that apply to images
 * @return array of chain names
 */
function _chain_list($applies_to = FALSE){
  global $theme;
  
  $chains = array();
  
  // Get theme chains
  $theme_hook = $theme.'_chain';
  if (function_exists($theme_hook)) {
    $theme_chains = $applies_to
      ? $theme_hook('list', $applies_to)
      : $theme_hook('list');
    if ($theme_chains) {
      $chains['theme'] = $theme_chains;
    }
  }

  // Get the module chains
  if ($modules = module_implements('chain')){
    foreach($modules as $module){
      
      $module_chains = $applies_to
        ? module_invoke($module, 'chain', 'list', $applies_to)
        : module_invoke($module, 'chain', 'list');
      
      if ($module_chains){
        $chains['modules'] = $chains['modules'] 
          ? array_merge($chains['modules'], $module_chains) 
          : $module_chains;
      }
    }
  }
  
  return $chains;
}


/**
 * Builds numeric options for dropdowns
 *
 * @param $start
 *  first option shown
 * @param $end
 *  last option shown
 * @param $step
 *  size of the increment
 * @param $alternate_label
 *  label of the alternate option, if applicable
 * @return unknown
 */
function _numeric_options($start, $end, $step, $alternate_label = FALSE){
  $options = array();
  for ($a = $start; $a <= $end; $a = $a + $step) {
    $options[(string)$a] = $a;
  }
  if (!$options[(string)$end]){
    $options[(string)$end] = $end;
  }
  
  if ($alternate_label){
    $options['#alternative'] = $alternate_label;
  }
  return $options;
}



/**
 * Loads a list of fonts made available by the painter as well as the theme
 *
 * @return unknown
 */
function _fonts(){
  $font_options = array();
  $dirs = array(
    'Painter fonts' => drupal_get_path('module', 'painter').'/fonts',
    'Theme fonts' => path_to_theme().'/fonts',
  );
  foreach($dirs as $label=>$dir){
    if (file_exists($dir)){
      $dir_handle = opendir($dir);
      while ($filename = readdir($dir_handle)){
        if (!is_dir($filename) && (strtolower(substr($filename, count($filename) - 5)) == '.ttf')){
          $font_options[$label][$filename] = substr($filename, 0, strlen($filename) - 4);
        }
      }
    }
  }
  return $font_options;
}



/**
 * Clears the cache of a particular chain
 * @param $chain_name
 */
function _clear_chain_cache($chain_name){
  $painter_cache = painter_cache();
  if ($painter_cache[$chain_name]){
    foreach($painter_cache[$chain_name] as $variant){
      print $variant['filename']."<br/>";
      file_delete($variant['filename']);
    }
    unset($painter_cache[$chain_name]);
  }
  painter_cache($painter_cache);
}



/**
 * Logs painter errors
 */
function painter_error($message = 'Error!'){
  painter_log($message, TRUE);
}



/**
 * Logs painter activities to the watchdog log
 *
 * @param $message
 *  message to log
 *  If omitted, the logs acumulated thus far is written to the watchdog log  
 * @param $error TRUE if an error is being logged
 */
function painter_log($message = FALSE, $error = FALSE){
  static $log = '';
  static $has_error = FALSE;
  
  if (!$message && !$error){
    if ($log) {
      if (variable_get('painter_watchdog', 0)){
        watchdog(
          'Graphics',
          $log,
          ($has_error ? WATCHDOG_ERROR : WATCHDOG_NOTICE)
        );
      }
      if (variable_get('painter_development', 0)){
        drupal_set_message($log);
      }
    }
    $log = '';
    $has_error = FALSE;
  }
  
  if ($error) {
    $has_error = TRUE;
    $log .= '<div class="error"><span>Error: </span>';
  } else {
    $log .= '<div class="info">';
  }
  
  $log .= $message.'</div>';
}



function painter_cache($cache = FALSE){
  static $painter_cache = FALSE;
  
  if ($cache) {
    $painter_cache = $cache;
    variable_set('painter_cache', $painter_cache);
  }
  
  if (!$painter_cache) {
    $painter_cache = variable_get('painter_cache', array());
  }
  
  return $painter_cache;
}